% This code reproduces the empirical illustration of 
% "A new approach to dating the reference cycle"
% by Maximo Camacho, Lola Gadea, and Ana Gomez-Loscos (2018)
% Estimates a mixture multiple-point change model for bivariate collection of peak-troug turning poins
% The authors have used and modify some functions of Sylvia Fruehwirth-Schnatter. 
% of her toolbox for Matlab "Finite Mixture and Markov Switching Models", bayesf Version 2.0
% Note: Bivariate ksdensity does not work for old MATLAB versions

clear
format bank
rng('default'); % This fixes the seed
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                      Please, adjust the paths                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
matlabrc; % clear previous paths and restore the original one
p1 = genpath('C:\aa\Empirical\general_functions');
path(path,p1) %general auxiliary matlab functions
p2 = genpath('C:\aa\Empirical\functions_empirical');
path(path,p2) %specific matlab functions of the empirical section

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Adjust any of the following to control specification desired %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
r=2;        % Number of characteristics (peaks and troughs) 
K=8;        % Number of clusters or groups in the mixture
M=2000;     % Number of draws from posterior
burnin=500; % Number of draws to be discarded
storeS=M;   % Only the last storeS allocations are stored
mcmcout.entropy=zeros(M,1);   % Stores EN(theta(m)/y)
mcmcout.xi=zeros(M,K,K);      % Stores transition probabilities
mcmcout.mu=zeros(M,r,K);      % Stores means
mcmcout.sigma=zeros(M,r*(r+1)/2,K);   % Stores variances
mcmcout.log_mixlik=zeros(M,1);        % Stores log p(y/theta(m))
mcmcout.log_mixprior=zeros(M,1);      % Stores log p(y/theta(m))?????????
mcmcout.logdet=zeros(M,K);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%            Reading the data and building file                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[data]=xlsread('data_nber.xlsx','NBER','A1:K622');
% Ten monthly coincident indicators used by the Business Cycle Committee when determining the trough of the Great Recession in the US. Available at 
% mirror.nber.org/cycles/BCDCFiguresData100920 ver5.xls
X=data;
dates=data(1:end-1,1);
X=X(1:size(X,1)-1,2:end);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                      CALL BRY-BOSCHAN                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[t n]=size(X);
STATES=NaN(t,n);
PEAKS=NaN(t,n);
TROUGHS=NaN(t,n);
for i=1:n
    clc;
    fprintf('Bry-Boschan to series %4.0f of %4.0f\n',i,n);
    x=X(:,i);
    [l1, l2]=position_nan(x);
    x= x(~isnan(x));
    l=length(x);
    [states,peaks,troughs,index_peaks, index_troughs] = bryboschan_me(x,1);
    STATES(l1:l2,i)=states;
    PEAKS(l1:l2,i)=index_peaks;
    TROUGHS(l1:l2,i)=index_troughs;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                      OUTPUT FILES   of BB algorithm     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% xlswrite('Peaks.xlsx', {'date'},1,'A1'); 
% xlswrite('Peaks.xlsx',1:1:n,1,'B1');
% xlswrite('Peaks.xlsx',dates,1,'A2');
% xlswrite('Peaks.xlsx',PEAKS,1,'B2');

% xlswrite('Troughs.xlsx', {'date'},1,'A1'); 
% xlswrite('Troughs.xlsx',1:1:n,1,'B1');
% xlswrite('Troughs.xlsx',dates,1,'A2');
% xlswrite('Troughs.xlsx',TROUGHS,1,'B2');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                 Building bi-variate data                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Build pairs of peaks/troughs or troughs/peaks from BB index of peaks and trougs
% Start returns P if the economy starts by a peak and T if starts by a trough
[PAIR, start]=preparing_file_bi(PEAKS,TROUGHS,dates); 
if start==1
    fprintf('THE ECONOMY STATRS BY A PEAK')
elseif start==2
    fprintf('THE ECONOMY STATRS BY A TROUGH')
end
Y=PAIR;
save('Y_NBER.dat','Y','-ASCII'); % Save the pairs for posterior analysis
N=size(Y,1);  % number of obs.
mcmcout.r=r;
mcmcout.K=K;
mcmcout.M=M;
mcmcout.S=zeros(storeS,N);       % Stores classifications S(m)
mcmcout.clust.prob=zeros(K,N);
mcmcout.clust.Smap=zeros(1,N);
mcmcout.clust.risk=zeros(1,N);
mcmcout.clust.Sident=zeros(1,N);
mcmcout.clust.Ssim=zeros(1,N);
yp=Y';
load nber_PT.dat % Read the NBER chronology (used as reference but omitted in computations)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%         Independent non-hierarchical priors                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%              Uninformative priors                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
b0=zeros(r,K);  % Priors for b0
B0=NaN(r,r,K);  
B0inv=NaN(2,2,K);
C0=NaN(r,r,K);  % Priors for B0, C0
for k=1:K
   B0k=1000000*eye(r);
   B0(:,:,k)=B0k;
   B0inv(:,:,k)=inv(B0k);
   C0(:,:,k)=eye(r);
end

% Prior variances inv(sigma)~W(c0,C0). 
c0=repmat(r,1,K);   % 1xK priors of c     
logdetC=zeros(1,K); 
for k=1:K
   logdetC(k)=log(det(C0(:,:,k)));
end

% Prior of the transition matrix  
ep=6;et=1/10;
p=kron(ep,eye(K));  
q=kron(et,eye(K)); 
e0=p+[zeros(K,1),q(:,1:end-1)]; 
etha0=zeros(K,K);               % KxK matrix of transition probs (rows sum to one)
for i=1:K
  gam=gamrnd(e0(i,:),1);
   while i<K && gam(1,i+1)<=0.01
   gam=gamrnd(e0(i,:),1);
   end
   etha0(i,:)=gam./repmat(sum(gam,2),1,K); % 1xK dranws of each row of etha
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% First classification S0 and initial value mu1_0 by K-modes   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
if start==1 %start with peaks
dis=24;         % minimun distance between peaks/trough
elseif start==2 % start with troughs
dis=24;         % minimun distance between trough/peak
end
[Mod,mukmod1,mukmod2,covarkmod,Skmod]=Kmodes_bi_empirical(Y,K,start,dis);
mu1=[mukmod1; mukmod2];
S1=Skmod'; %Initial classification

% Defining different WARNINGS to detect unwished situations
% Warning1 counts the percentage of draws in which the algorithm has been unable to generate a bivariate vector of mu that complies with the partition constraints of the reference cycle; 
% Warning2 checks if the estimated mu values are within the range of data values allowing a margin of 1 year at the beginning and end; 
% Warning3 counts the percentage of classifications that leave an empty cluster.
warning1=zeros(M,1);
warning2=0;
warning3=zeros(M,1);
index_bad=[]; % Draws that does not comply the reference cycle restrictions

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                   Running MCMC algorithm                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Version 2015a Matlab
for m=1:(burnin+M)
  clc;
  fprintf('MCMC number %4.0f of %4.0f\n',m,burnin+M);
  mcmcout.ST(m,:)=S1;  % Store classifications
  % 1) Chib(1998) pkk~Beta(ek1,ek2) => pkk/S~Beta(ek1(S),ek2(S)), k=1,...,K-1
  %    where ek1(S)=ek1+Nkk(S), and ek2(S)=ek2+Nkk+1(S) and Nij(S)=No{St=j/St-1=i}
  %    Note that Nkk+1(S)=1
  Nij=zeros(K);
  for tr=[S1(1:end-1);S1(2:end)]
    Nij(tr(1),tr(2))=Nij(tr(1),tr(2))+1;
  end
  Nij_nerg=diag(diag(Nij));
  for k=1:K-1
      Nij_nerg(k,k+1)=1;
  end
  e1=Nij_nerg+e0;
  etha1=zeros(K,K);   % KxK matrix of transition probs (rows sum to one)
  for i=1:K
      gam=gamrnd(e1(i,:),1);
      etha1(i,:)=gam./repmat(sum(gam,2),1,K); % 1xK dranws of each row of etha
  end
  etha1=check_etha(etha1); % Round-off errors
  ind=repmat(S1,K,1)==repmat([1:K]',1,N);
  Nk=sum(ind,2)';  % This is 1xK vector Nk(s) 
   
  % 2) Sample variances given means. If iv2~W[c,C] => iv2/mu,S~W[ck(S),Ck(S)]
  %    where ck(S)=c+Nk(S) and Ck(S)=C+sumk(Yk-muk)(Yk-muk)'           
  c1=c0+Nk;
  C1=NaN(r,r,K);
  sigma1=NaN(r,r,K);
  sigma1inv=NaN(r,r,K);
  for k=1:K
     b=(yp(:,S1==k)-repmat(mu1(:,k),1,Nk(k)));
     C1(:,:,k)=C0(:,:,k)+b*b';
     sigma1(:,:,k)=inv(wishrnd(inv(C1(:,:,k)),c1(k)));   % 2x2 dranws of covariance matrix 
     sigma1inv(:,:,k)=inv(sigma1(:,:,k));
     logdetsigma1(k)=log(det(sigma1(:,:,k)));
     logdetinv(k)=log(det(sigma1inv(:,:,k)));
  end 
  
  % 3) Sample means given variances p. 178. If mu~N[b,B] => mu/v2,S~N[bk(S),Bk(S)]
  %    where inv(Bk(S))=inv(B)+inv(v2)*Nk(S) and bk(S)=Bk(S)*[inv(v2)*Nk(S)*mean(y/S)+inv(B)]*b] 
  %    We employ a rejection sampling approach peak(k)<trough(k)<peak(k+1)
  for k=1:K
     B1inv(:,:,k)= B0inv(:,:,k)+Nk(k)*sigma1inv(:,:,k); 
     B1(:,:,k)=inv(B1inv(:,:,k));
     b1(:,k)=B1(:,:,k)*(squeeze(B0inv(:,:,k))*b0(:,k)+sigma1inv(:,:,k)*sum(yp(:,S1==k),2));   
  end
   
j=1;
while j<1000
  for k=1:K
      mu1(:,k)=b1(:,k)+chol(B1(:,:,k))'*randn(size(B1(:,:,k),1),1);
      while mu1(1,k)>mu1(2,k) % Check peaks before troughts
      mu1(:,k)=b1(:,k)+chol(B1(:,:,k))'*randn(size(B1(:,:,k),1),1);
     end
  end
  result=check_RC(mu1);
  if result==0
      break
  else
      j=j+1;
  end
end
if j==1000
    display('With these data and parameters it is not possible to guarantee the identification of the reference cycle')
    warning1(m)=1;
    index_bad=[index_bad;m];
end


  % 4) Sample classifications given means, variances by sampling S from p[s/theta,y]~p(y/S,theta)*p(S/theta)
  %    4.1) Run the Hamilton filter => compute filtered probabilities
  %               Pr(St/yt,theta) for t=1...K and k=1,...,K
  %      If Y~N[m,V] => f=(2*pi)^(-.5*n)*det(V)^(-.5)*exp[-.5*(Y-m)'inv(V)*(Y-m)]
  llh=zeros(K,N); 
  llh1=-.5*r*(log(2*pi)); 
  for k=1:K
      eps=yp-mu1(:,k*ones(1,N)); 
      llh(k,:)=llh1-.5*logdetsigma1(k)-.5*sum(eps'*squeeze(sigma1inv(:,:,k)).*eps',2)';   
  end
  lh=exp(llh);                   % Likelihood p[y/S,theta(k)]
  p_filt=zeros(K,N+1);           % Kx(N+1) vector of filtered probabilities
  pinit=[1;zeros(K-1,1)];        % Ininial (it colud be ergodic or 1/K)
  p_filt(:,1)=pinit;               
  mixlik=0;
  for t=1:N
      p=(etha1'*p_filt(:,t)).*lh(:,t);  % Numerator, Kx1 vector
      st=sum(p,1);                      % Denominator 
      mixlik=mixlik+log(st);            % sum loglike mixture
      p_filt(:,t+1)=p/st;               % filtered (classification) probabilities at t, Kx1 vector Pr(St/yt,theta)
  end 
 
  %    4.2) Sample ST(m) at T from Pr(ST/yT,theta) (Multi-move sampling)
  S1=ones(N+1,1);  
  S1(N+1,1)=sum(cumsum(p_filt(1:K-1,N+1))<rand)+1;    
  pd=log(p_filt(S1(N+1,1),N+1));    % log(p(S/theta))
  

  %    4.3) For t=T-1,T-2,...1 sample St(m) from Pr(St/St+1,yt,theta)  
  for t=N:-1:1      
    p=p_filt(:,t).*etha1(:,S1(t+1,1)); 
    p=p/sum(p); 
    S1(t,1)=sum(cumsum(p(1:K-1))<rand)+1; 
    pd=pd+log(p(S1(t,1)));  
  end

  S1=S1(2:N+1)';            % 1xN vector of posterior allocations
  p_filt=p_filt(:,2:N+1)';  % NxK matrix of filtered probabilities 

  % 5) Permutation (optional)
   %The permutation sampler described by Fruhwirth-Schnatter (2001) that consists on reordering the
  %values of the draws in order to fulfill the state-identifying restrictions does not apply easily in this multivariate framework

  
  % 6) Pg. 210: Bayesian Maximum A Posteriori (MAP) classification:  mcmcout.clust.Smap is S1 maximizing the joint posterior p(theta,S|y)=p(y/theta,S)*p(S/theta)*p(theta)
  
  % Calculate mixprior
  % log(p(theta))=log(p(mu,sigma,theta))=log(p(mu))+log(p(sigma))+log(p(theta))
  
  % log(p(mu)) from log multivariate normal densities mu~N[b0,B0]
  % If Y~N[m,V] => f=(2*pi)^(-.5*n)*det(V)^(-.5)*exp[-.5*(Y-m)'inv(V)*(Y-m)]
  par_logdetinv=zeros(K,1); 
  for k=1:K
     par_logdetinv(k)=log(det(squeeze(B0inv(:,:,k))));  
  end  
  err=repmat(mu1-b0,[1 1 r]);
  errmat=permute(err,[1 3 2]).*permute(err,[3 1 2]);
  fd1=-0.5*sum(r*log(2*pi)+par_logdetinv+squeeze(sum(sum(B0inv.*errmat,1),2)),1);  

  % log p(sigmainv) from log Wishart sigmainv~W[c,C] 
  C0b=manipulate_matrix(C0,2); 
  arg_yinvcol=zeros(r*(r+1)/2,K);
  for k=1:K
      arg_yinvcol(:,k)=manipulate_matrix(squeeze(sigma1inv(:,:,k)),1);
  end 
  index=0.5*[[1:r].*[2:r+1]]';
  trQS=2*sum(arg_yinvcol.*C0b,1)-sum(arg_yinvcol(index,:).*C0b(index,:),1);
  fd2=sum(c0.*logdetC+(c0+(r+1)/2).*logdetinv-trQS-r*(r-1)/4*log(pi),2);  
  fd2=fd2-sum(sum(gammaln(repmat(c0,r,1)+0.5-0.5*repmat([1:r]',1,K)),1),2);
     
  % p(mu,sigma)=p(sigma)*p(mu) => fd=log(p(theta))=log(p(sigma))+log(p(mu))
  fd=fd1+fd2;
  
  % log(etha) from log Gamma pag. 432 (expression A.5)
  fl=gammaln(sum(e0,2))+sum((e0-1).*log(etha1)-gammaln(e0),2);
    
  % logprior=log(theta)=log(p(mu,sigma,etha))=log(p(sigma))+log(p(mu))+log(etha)
  logprior=fd+sum(fl); 
  mixprior=logprior;

  % log-Posterior: log(p(theta/y,S))=log(p(y/theta,S)+log(p(theta))
   mixpost=mixlik+mixprior;  
              
  % mcmcout.clust.Smap is S1 that maximizes log(p(theta,S|y))
  if m>burnin
    % Store log of complete data posterior: log(p(theta,S|y))=log(p(y/theta,S))+log(p(S/theta))+log(p(theta)) 
    %mcmcout.log_cdpost(m-burnin)=mixpost+pd;   
    mcmcout.log_cdpost(m-burnin)=mixlik+pd;  
    if (m-burnin)==1  
        maxcdpost=mcmcout.log_cdpost(1)-1;   
    end
    if mcmcout.log_cdpost(m-burnin)> maxcdpost
        mcmcout.clust.Smap=S1;
    end
  end
 
  
  % 7) Compute entropy 
  logp=zeros(size(p_filt));
  for k=1:K
      logp(p_filt(:,k)==0,k)=-99;
      logp(p_filt(:,k)~=0,k)=log(p_filt(p_filt(:,k)~=0,k));
  end
  entropy=-sum(sum(logp.*p_filt));

  % Store all the results
  if m>burnin 
     mcmcout.entropy(m-burnin)=entropy;
     mcmcout.log_mixlik(m-burnin)=mixlik;               % Stores log[p(y/theta(m))]
     mcmcout.log_mixprior(m-burnin)=mixprior;           % Stores the log of the prior log[p(theta(m))]
     mcmcout.log_mixpost(m-burnin)=mixpost;
     mcmcout.xi(m-burnin,:,:)=etha1;                    % Stores trans. probs
     mcmcout.mu(m-burnin,:,:)=mu1;                      % Stores means
     mcmcout.sigma(m-burnin,:,:,:)=manipulate_matrix(sigma1,2);  % Stores different elements of VARCOV
     mcmcout.logdet(m-burnin,:)=logdetinv;              % Stores log(det((inv(sigma)))
     mcmcout.weight(m-burnin,:)=marstat(etha1);         % Compute ergodic probabilities from etha1
     
    if  m>(M+burnin-storeS)  
        mcmcout.S(m-(M+burnin-storeS),:)=S1;            % Stores classifications S(m)     
    end
  end
end

index_bad=index_bad-burnin;
index_bad(index_bad<=0)=[];
mcmcout.entropy(index_bad,:)=[];
mcmcout.log_mixlik(index_bad,:)=[];
mcmcout.log_mixprior(index_bad,:)=[];
mcmcout.log_mixpost(:,index_bad)=[];
mcmcout.xi(index_bad,:,:)=[];
mcmcout.mu(index_bad,:,:)=[];
mcmcout.sigma(index_bad,:,:)=[];
mcmcout.logdet(index_bad,:)=[];
mcmcout.weight(index_bad,:)=[];
M2=M-length(index_bad);


% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %                       Identification                         %
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %Clustering mcmcout.clust.Smap without identification, is computed within the MCMC
 mcmcout.clust.Ssim=compute_similarity_matrix(mcmcout.S,K); % Similarity matrix
 
 % identify the mixture model before performing clustering in order to ensure that the
 % labelling is the same
clust.prob=zeros(K,N);


    for k=1:K
        clust.prob(k,:)=sum(mcmcout.S==k,1)/size(mcmcout.S,1) ;
    end

    [~,index]=max(clust.prob'); %re-arrange clust.prob by date
    [~,index2]=sort(index);
    clust.prob=clust.prob(index2,:);
   
    [pmax Saverage]=max(clust.prob);
    clust.risk=1-pmax;
    clust.Sident=Saverage;  

%[Y,mcmcout.clust.Smap',mcmcout.clust.Ssim',mcmcout.clust.Sident',mcmcout.clust.risk',mcmcout.clust.prob']
mcmcout.clust.prob=clust.prob;
mcmcout.clust.risk=clust.risk;
mcmcout.clust.Sident=clust.Sident;
%[mcmcout.clust.Smap',mcmcout.clust.Sident',mcmcout.clust.Ssim']
mcmcout.clust.Smap=arrange_S(mcmcout.clust.Smap);
mcmcout.clust.Sident=arrange_S(mcmcout.clust.Sident);
mcmcout.clust.Ssim=arrange_S(mcmcout.clust.Ssim);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Information criteria
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
mcmcout.log_mixlik= mcmcout.log_mixlik(~isnan(mcmcout.log_mixlik)); %remove NaN elements
mcmcout.entropy= mcmcout.entropy(~isnan(mcmcout.entropy)); %remove NaN elements
[logm is]=sort(mcmcout.log_mixlik);  % approximate ML estimator
loglikmax=logm(end);
mldraw=is(end);
entropy_ml=mcmcout.entropy(mldraw);
dk=(K-1)+2*K+3*K; %num parameters, (K-1) transition matrix, 2*K means and 3*K covariance matrix
ic.loglikmax=loglikmax;
ic.entropy_ml=entropy_ml;
ic.aic= -2*loglikmax+2*dk;
ic.bic= -2*loglikmax+log(N)*dk;
%ic.bicmc=-2*mean(mcmcout.log.mixlik)+dk*(log(N)-1);
ic.iclbic_ml= ic.bic+2*entropy_ml;
mcmcout.ic=ic;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                   Parameter estimation                       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc;
if start==1
    P=squeeze(mcmcout.mu(:,1,:));
    T=squeeze(mcmcout.mu(:,2,:)); 
    MU=[mean(P)',mean(T)'];
  
    fprintf('THE ECONOMY STATRS BY A PEAK\n')
elseif start==2
    P=squeeze(mcmcout.mu(:,2,:));   
    T=squeeze(mcmcout.mu(:,1,:));
    MU=[mean(T)',mean(P)'];
    nber_PT=fliplr(nber_PT);
    nber_PT=[nber_PT(1:end-1,1),nber_PT(2:end,2);NaN,NaN]
    fprintf('THE ECONOMY STATRS BY A TROUGH\n')
end
[MU, index]=sort_matrix_ascend(MU,1);
if min(min(MU))<min(min(Y))-1 || max(max(MU))>max(max(Y))+1
    display('Results are out of the data range')
end
MU=decimals_to_months_matrix(MU);
b0=decimals_to_months_matrix(b0);

if min(min(MU))<min(min(Y))-1 || max(max(MU))>max(max(Y))+1
    display('Results are out of the data range')
    warning2=1;
end
warning1=sum(warning1)/(M+burnin);
mcmcout.warning.one=warning1;
mcmcout.warning.two=warning2;

for i=1:M2
    if max(mcmcout.S(i,:))<K
        warning3(i)=1;
    end
end
warning3=sum(warning3)/M2;
mcmcout.warning.three=warning3;

% VARCOV
VARI=zeros(K,(r^2-r)/2+r);
for it=1:K
    VARI(it,:)=mean(mcmcout.sigma(:,:,it));
end
VARI=VARI(index,:); %we order VARI groups according with mu groups

%transition matrix
pshi=squeeze(mean(mcmcout.xi));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                   Here some output results                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc;
fprintf('Table 4: Model estimation \n')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%Printing main results%%%%%%%%%%%%%
disp('                NBER                         b0                          mu');
disp([nber_PT,b0',MU]);

%Calculate CI
z=norminv(0.025);
decimals_to_months_matrix(VARI);
for k=1:K
    mu1=MU(k,1);
    C1=mcmcout.mu(:,1,k);
    C1=sort(C1);
    IC1(k,1)=C1(M*0.0275);
    IC1(k,2)=C1(M*0.975);
    C2=mcmcout.mu(:,2,k);
    C2=sort(C2);
    IC2(k,1)=C2(M*0.0275);
    IC2(k,2)=C2(M*0.975);
end

disp('Confidence intervals for peaks:')
disp(decimals_to_months_matrix(IC1))
disp('Confidence intervals for trougs:')
disp(decimals_to_months_matrix(IC2))

% disp('The covariate matrix is');
% disp(VARI);
% 
% disp('The transition matrix is:');
% disp(pshi);

disp('     Log-likelihood     AIC          BIC          Entropy      BIC+Entropy');
disp([mcmcout.ic.loglikmax mcmcout.ic.aic mcmcout.ic.bic mcmcout.ic.entropy_ml mcmcout.ic.iclbic_ml]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                       Plots of the output
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Figure 1
ifig=1;
axes1 = axes('Parent',figure(ifig)); % Create axes
hold(axes1,'on');
band=0.8;
ksdensity(Y,'Bandwidth',band) % Create surf
title('Figure 1: Bivariate distribution of specific turning point dates'); % Create title
view(axes1,[-37.5 30]);
grid(axes1,'on');
set(axes1,'XDir','reverse'); % Set the remaining axes properties

%Figure2 a
ifig=ifig+1; figure(ifig);
%Point representation of the means and variances
  indexscatter=fix(linspace(1,M,min(2000,M)));
  sizep=2;cp='ko';%
  cp='rx';% cp='go';
  r=size(mcmcout.mu,2);
  indexdiag=[1:fix(r*(r+1)/2)]';        
  indexdiag=diag(manipulate_matrix(indexdiag,3))';     
    
 subplot(1,r,1)
        for j=1:K
            scatter(squeeze(mcmcout.mu(indexscatter,1,j)),squeeze(mcmcout.sigma(indexscatter,indexdiag(1),j)),sizep,cp);
            hold on;
            text(mean(squeeze(mcmcout.mu(indexscatter,1,j))),mean(squeeze(mcmcout.sigma(indexscatter,indexdiag(1),j))),num2str(j));
        end
        xlabel('\mu_{k}^{P}');
        ylabel('\Sigma_{11,k}');
        subplot(1,r,2)
        for j=1:K
            scatter(squeeze(mcmcout.mu(indexscatter,2,j)),squeeze(mcmcout.sigma(indexscatter,indexdiag(2),j)),sizep,cp);
            hold on;
            text(mean(squeeze(mcmcout.mu(indexscatter,2,j))),mean(squeeze(mcmcout.sigma(indexscatter,indexdiag(2),j))),num2str(j));
        end
        xlabel('\mu_{k}^{T}');
        ylabel('\Sigma_{22,k}');
        suptitle('Figure 2: Scatter plots of means and variances. Panel B.')
        
    %Figure 2 b point process for the means of each bivariate density
    
    ifig=ifig+1;
    figure(ifig);     
    ij=nchoosek([1:r]',2);
    [nr, nc]=plotsub(size(ij,1));
    
    for ii=1:size(ij,1)
                subplot(nr,nc,ii);  
        for j=1:K
            scatter(squeeze(mcmcout.mu(indexscatter,ij(ii,1),j)),squeeze(mcmcout.mu(indexscatter,ij(ii,2),j)),sizep,cp);
         hold on;
         text(mean(squeeze(mcmcout.mu(indexscatter,ij(ii,1),j))),mean(squeeze(mcmcout.mu(indexscatter,ij(ii,2),j))),num2str(j));

        end
        
        xlabel('\mu_{k}^{P}');
        ylabel('\mu_{k}^{T}');
        suptitle('Figure 2: Scatter plots of means and variances. Panel A.')
    end
    
%Figure 3a Posterior draws of mu
for j=1:r
        ifig=ifig+1;figure(ifig);
        plot(squeeze(mcmcout.mu(:,j,:))); 
        legend([repmat('k=',K,1) num2str([1:K]')]);
        title(['Figure 3: Behaviour of Gibbs sampler. Draws of \mu_{k,' num2str(j) '} of feature ' num2str(j)]);
end

%Figure 3 b: Posterior draws of Sigma
ifig=ifig+1;figure(ifig);
    plot(-mcmcout.logdet);
    legend([repmat('k=',K,1) num2str([1:K]')]);
    title(['Figure 3: Behaviour of Gibbs sampler. Draws of log(det(\Sigma_k))']);

%Figure 3c: Posterior draws of transtion matrix
    ifig=ifig+1;figure(ifig);
            for k=1:K
                for j=1:K
                    subplot(K,K,(k-1)*K+j); 
                    plot(mcmcout.xi(:,k,j));
                    axis([0 mcmcout.M 0 1])
                end
            end
            subplot(K,K,2); 
            suptitle(['Figure 3: Behaviour of Gibbs sampler. Draws of transtion matrix']);     
    
%Figure 4: Clustering figure   
        if start==1 %Economy starts by peak
            yp=Y(:,1);
            yt=Y(:,2);
        else star==2 %Economy starts by trought
            yp=Y(:,2);
            yt=Y(:,1);
        end
        [yps is]=sort(yp);
        [yts is]=sort(yt);

           ifig=ifig+1;
           figure(ifig);
            for k=1:K
            subplot(K,1,k);
            scatter(yps,mcmcout.clust.prob(k,is)');
            hold on 
            scatter(yts,mcmcout.clust.prob(k,is)');
            hold off
            %if k==1 title('Estimated Classification Filtered Probabilities'); end
            axis([min(yp)-1 max(yt)+1 0 1]);
            %xlabel('observation y_i');
            %ylabel(['Prob(S_i=',num2str(k),'|\theta ,\tau ^{i})'],'fontsize',8);
            end
            legend('peaks','troughs')
        suptitle(['Figure 4: Classification probabilities']); 
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                       Making a Table with the results
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%            
DEV=count_months(MU, nber_PT);
matrix=[nber_PT,MU,DEV];
file=strcat('Table_results_NBER_series.tex');
fid=fopen(file,'w');
tt=['\\begin{table}\\caption{Results of empirical illustration (NBER series)}\\label{tab:1}\\begin{center}\\scalebox{1}{\\begin{tabular}{cc|cc|cc} \\hline \\hline \n'];
fprintf(fid,tt);
tt=['\\multicolumn{2}{c}{NBER}' '&' '\\multicolumn{2}{|c}{MSMM}'   '&' '\\multicolumn{2}{|c}{Deviation (in months)}' '\\\\ \\hline \n'];
fprintf(fid,tt);
tt=['Peaks ' '&' 'Troughs'  '&' 'Peaks'   '&' 'Troughs'   '&' 'Peaks'   '&' 'Troughs'  '\\\\ \\hline \n'];
fprintf(fid,tt);

for i=1:K
    tt=[];
    for j=1:6
    tt=[tt,'&' ,num2str(matrix(i,j),'%5.2f') ];
    end   
    tt=[tt,'\\\\ \n'];
    tt=tt(2:length(tt));
    fprintf(fid,tt);
end

tt='\\hline \\hline \\end{tabular}}\\end{center}\\end{table}\n';
fprintf(fid,tt);
st=fclose(fid);

%save output
save('MCMCOUT_empirical.mat','mcmcout')

